/* ANCHOR: all */
# Ceremony Repository

// ANCHOR: content
This repository holds data pertaining to ceremonies. The primary data consists of:

* Transaction proposals

* Transaction approvals

* Tamper proofing evidence

* Policies (such as spending rules)

* Trusted PGP keys

* Participants

## Directives

* MUST be a private repository

* MUST be write protected, requiring approval from at least 1 individual other than one who opened the PR for merging

* MUST require signed commits

## Repository Structure

```
ceremonies/
    <date>/
        log.txt
            - [ ] TODO: write a layout for the log
        tamper_evidence/
            <photo_name>.jpeg
            <photo_name>.jpeg
        transactions/
            <tx_name>.tx.json
policies/
    spending-policy.json [NOT IMPLEMENTED]
keychain/
    <key_fingerprint>/
        <last_16_digits_of_key_fingerprint>.asc
```

## Procedure: Setting up Repository

{{ #include ./git-repository-initialization.md:procedure}}

## Procedure: Adding a OpenPGP Public Certificate

1. Designate the role of the key - it should be placed into the corresponding role directory

1. Open a PR submitting the key to the repository

    * MUST be via commit signed by the PGP key being submitted to the repository

1. Two other authorized individuals (TODO define how they are authorized) must perform a signing ceremony where the master key signature of the public certificate is added to the public certificate

    - [ ] TODO add instructions/doc

1. The PR should be merged using a signed commit via the git CLI

## Procedure: Adding Scripts

### Script: Verify OpenPGP Certificate Signatures (`verify-openpgp-certificates.sh`)

This script is used during ceremonies where operators want to ensure that a set of OpenPGP certificates are signed by each of the operators at least once. The way it is used is:

1. Plugs in the Ceremony SD card

1. The operator plugs in their smart card which holds their OpenPGP subkeys

1. Runs the `verify-openpgp-certificates.sh`

#### Provisioning

1. Create a file called `verify-openpgp-certificates.sh` in the `scripts/` directory

1. Add the following contents to the file:
- [ ] TODO, modify to check signatures on the pub key 
- [ ] TODO review script
    ```
    #!/bin/bash

    set -eu -o pipefail

    DIRECTORY="$1"

    if ! compgen -G "$DIRECTORY/*.asc" > /dev/null; then
    echo "No .asc files found in the directory."
    exit 1
    fi

    if ! gpg --card-status > /dev/null 2>&1; then
    echo "No smart card detected. Please insert a smart card."
    exit 1
    fi

    smart_card_id=$(gpg --card-status | grep sec | cut -d'/' -f2 | cut -d' ' -f1)

    for asc_file in "$DIRECTORY"/*.asc; do
    gpg --import "$asc_file"
    done

    for asc_file in "$DIRECTORY"/*.asc; do
    sig_exists="false"

    for sig_file in "$DIRECTORY"/*.asc.sig; do
        sigfile_basename=$(basename "$sig_file" .asc.sig)
        ascfile_basename=$(basename "$asc_file" .asc)

        if [[ "$sigfile_basename" != "$ascfile_basename" ]]; then
            continue
        fi

        sig_key_id=$(gpg --verify "$sig_file" 2>&1 | grep 'Primary key fingerprint' | cut -d' ' -f4- | tr -d ' ')
        trimmed_key_id="${sig_key_id: -16}"

        if [[ "$trimmed_key_id" == "$smart_card_id" ]]; then
            sig_exists="true"
            break
        fi
    done

    if [[ "$sig_exists" == "false" ]]; then
        printf "\nWARNING: Signature for %s by operator key %s does not exist\n" "$asc_file" "$smart_card_id"
    fi
    done

    ```

### Script: Verify Workflow Payload Has Valid OpenPGP Signatures (`verify-workload-payloads.sh`)

This script is used during ceremonies to ensure that the payload data from the "Proposer" and "Approvers" have been signed by trusted keys from the `keychain/` directory. The script `verify-openpgp-signatures.sh` is used to load and verify the validity of keys before this script can be used. 

1. Run the `verify-openpgp-certificates` scripts

1. Plug in the "Workflow" SD card and run the `verify-workload-payload.sh`

#### Provisioning

1. Create a file called `verify-workload-payload.sh` in the `scripts/` directory

1. Add the following content to the file:
- [ ] TODO: review script
    ```
    #!/bin/bash

    DIRECTORY=$1

    declare -a key_ids

    while IFS= read -r line; do
        key_id=$(echo "$line" | awk -F: '/^pub/{print $5}')
        if [[ -n "$key_id" ]]; then
            key_ids+=("$key_id")
        fi
    done < <(gpg --list-keys --with-colons)

    check_key_id() {
        local search_key_id="$1"
        for id in "${key_ids[@]}"; do
            if [[ "$id" == "$search_key_id" ]]; then
                return 0 
            fi
        done
        return 1 
    }

    for tx in "$DIRECTORY"/*.json; do
        basename=$(basename "$tx" .json)
        number_of_sigs=0
        tx_sig="$DIRECTORY/$basename.json.sig"

        if [[ ! -f "$tx_sig" ]]; then
            echo "WARNING: No signature file found for transaction $tx."
            continue
        fi

        sig_key_id=$(gpg --verify "$tx_sig" 2>&1 | grep 'Primary key fingerprint' | cut -d' ' -f4- | tr -d ' ')
        trimmed_key_id="${sig_key_id: -16}"

        if check_key_id "$trimmed_key_id"; then
            ((number_of_sigs++))
        else
            echo "Key ID $trimmed_key_id not found in key_ids array."
        fi    

        if (( number_of_sigs < 2 )); then
            echo "WARNING: Insufficient signatures ($number_of_sigs) for transaction $tx."
        fi
    done
    ```

## Procedure: Provision Ceremony SD Card

This procedure requires 2 individuals in order to witness the process and verify that the data being burned to the card is correct.

The Ceremony SD Card once provisioned will be used in creating the [tamper proofed airgap bundle](#air-gapped-bundle)

// ANCHOR: provision-ceremony-sd-card
1. Get a freshly formatted SD card 

1. Plug it into a computer

1. Navigate the the official Keychain repository of your organization

1. Select provisioner and approver keys from the Keychain repository

1. Download the desired keys along with detached signatures

1. Find the SD card block device name using `lsblk`

1. Create a directory for OpenPGP public certificates on the SD card: `mkdir dev/<device_name>/public_certificates`

1. Copy the `.asc` and `.sig` signature files from `keychain` directory in the Ceremonies repository to into the `public_certificates` dir on the SD card

1. Create a directory for scripts on the SD card: `mkdir dev/<device_name>/scripts`

1. Copy the contents of the scripts directory from the Ceremonies repository into the `scripts/` directory on the SD card

1. Use the `sdtool` to lock the card

{{ #include ../sdtool-instructions.md:steps }}

1. Label the card "Ceremony [date]"

### Tamper Proofing

{{ #include ./tamper-evidence-methods.md:vsbwf-procedure-sealing }}

// ANCHOR_END: provision-ceremony-sd-card

// ANCHOR_END: content
/* ANCHOR_END: all */

